@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders.c,v 1.141 2008/03/28 16:56:28 chu11 Exp $
+ *  $Id: genders.c,v 1.145 2009/05/21 17:52:50 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -90,7 +90,9 @@ _initialize_handle_info(genders_t handle)
   memset(handle->nodename,'\0',GENDERS_MAXHOSTNAMELEN+1);
   handle->valbuf = NULL;
   handle->node_index = NULL;
+  handle->node_index_size = 0;
   handle->attr_index = NULL;
+  handle->attr_index_size = 0;
   handle->attrval_index = NULL;
   handle->attrval_index_attr = NULL;
 
@@ -117,10 +119,9 @@ genders_handle_create(void)
   __list_create(handle->nodeslist, _genders_list_free_genders_node);
   __list_create(handle->attrvalslist, _genders_list_free_attrvallist);
   __list_create(handle->attrslist, free);
-  
-  /* node_index and attr_index created in genders_load_data after node
-   * and attr counts are determined.  Valbuf created in
-   * genders_load_data after maxvallen is calculated
+
+  /* node_index, attr_index created in genders_load_data.  Valbuf
+   * created in genders_load_data after maxvallen is calculated
    */
   
   handle->errnum = GENDERS_ERR_SUCCESS;
@@ -171,11 +172,31 @@ genders_load_data(genders_t handle, const char *filename)
 
   if (_genders_unloaded_handle_error_check(handle) < 0)
     goto cleanup;
+  
+  handle->node_index_size = GENDERS_NODE_INDEX_INIT_SIZE;
+  
+  __hash_create(handle->node_index,
+                handle->node_index_size,
+                (hash_key_f)hash_key_string,
+                (hash_cmp_f)strcmp, 
+                NULL);
+  
+  handle->attr_index_size = GENDERS_ATTR_INDEX_INIT_SIZE;
+  
+  __hash_create(handle->attr_index,
+                handle->attr_index_size,
+                (hash_key_f)hash_key_string,
+                (hash_cmp_f)strcmp, 
+                (hash_del_f)list_destroy);
 
   if (_genders_open_and_parse(handle, 
 			      filename, 
 			      handle->nodeslist, 
 			      handle->attrvalslist,
+                              &(handle->node_index),
+                              &(handle->node_index_size),
+                              &(handle->attr_index),
+                              &(handle->attr_index_size),
 			      0, 
 			      NULL) < 0)
     goto cleanup;
@@ -198,12 +219,6 @@ genders_load_data(genders_t handle, const char *filename)
   /* Create a buffer for value substitutions */
   __xmalloc(handle->valbuf, char *, handle->maxvallen + 1);
 
-  if (_genders_index_nodes(handle) < 0)
-    goto cleanup;
-
-  if (_genders_index_attrs(handle) < 0)
-    goto cleanup;
-
   handle->is_loaded++;
   handle->errnum = GENDERS_ERR_SUCCESS;
   return 0;
@@ -600,7 +615,7 @@ genders_getnodes(genders_t handle, char *nodes[], int len,
       /* Case B: atleast the attr was input, so use attr_index */
       List l;
       
-      if (!handle->attr_index)
+      if (!handle->numattrs)
         {
 	  /* No attributes, so no nodes have this attr */
 	  handle->errnum = GENDERS_ERR_SUCCESS;
@@ -670,7 +685,7 @@ genders_getattr(genders_t handle,
   if (!node)
     node = handle->nodename;
   
-  if (!handle->node_index)
+  if (!handle->numnodes)
     {
       handle->errnum = GENDERS_ERR_NOTFOUND;
       return -1;
@@ -773,7 +788,7 @@ genders_testattr(genders_t handle,
   if (!node)
     node = handle->nodename;
 
-  if (!handle->node_index)
+  if (!handle->numnodes)
     {
       handle->errnum = GENDERS_ERR_NOTFOUND;
       return -1;
@@ -834,7 +849,7 @@ genders_testattrval(genders_t handle,
   if (!node)
     node = handle->nodename;
 
-  if (!handle->node_index)
+  if (!handle->numnodes)
     {
       handle->errnum = GENDERS_ERR_NOTFOUND;
       return -1;
@@ -864,7 +879,7 @@ genders_isnode(genders_t handle, const char *node)
   if (!node)
     node = handle->nodename;
 
-  if (!handle->node_index)
+  if (!handle->numnodes)
     {
       /* No nodes, so node not found */
       handle->errnum = GENDERS_ERR_SUCCESS;
@@ -890,7 +905,7 @@ genders_isattr(genders_t handle, const char *attr)
       return -1;
     }
 
-  if (!handle->attr_index)
+  if (!handle->numattrs)
     {
       /* No attributes, so attr not found */
       handle->errnum = GENDERS_ERR_SUCCESS;
@@ -932,7 +947,15 @@ genders_isattrval(genders_t handle, const char *attr, const char *val)
     }
   else 
     {
-      __list_iterator_create(nodeslist_itr, handle->nodeslist);
+      List l;
+
+      if (!handle->numattrs)
+        goto out;
+
+      if (!(l = hash_find(handle->attr_index, attr)))
+        goto out;
+
+      __list_iterator_create(nodeslist_itr, l);
       while ((n = list_next(nodeslist_itr))) 
 	{
 	  if (_genders_find_attrval(handle, n, attr, val, &av) < 0) 
@@ -946,6 +969,7 @@ genders_isattrval(genders_t handle, const char *attr, const char *val)
 	}
     }
 
+ out:
   rv = 0;
   handle->errnum = GENDERS_ERR_SUCCESS;
  cleanup:
@@ -1092,6 +1116,10 @@ genders_parse(genders_t handle, const char *filename, FILE *stream)
   int errcount, rv = -1;
   List debugnodeslist = NULL;
   List debugattrvalslist = NULL;
+  hash_t debugnode_index = NULL;
+  int debugnode_index_size = GENDERS_NODE_INDEX_INIT_SIZE;
+  hash_t debugattr_index = NULL;
+  int debugattr_index_size = GENDERS_ATTR_INDEX_INIT_SIZE;
 
   if (_genders_handle_error_check(handle) < 0)
     goto cleanup;
@@ -1101,11 +1129,25 @@ genders_parse(genders_t handle, const char *filename, FILE *stream)
 
   __list_create(debugnodeslist, _genders_list_free_genders_node);
   __list_create(debugattrvalslist, NULL);
+  __hash_create(debugnode_index,
+                debugnode_index_size,
+                (hash_key_f)hash_key_string,
+                (hash_cmp_f)strcmp, 
+                NULL);
+  __hash_create(debugattr_index,
+                debugattr_index_size,
+                (hash_key_f)hash_key_string,
+                (hash_cmp_f)strcmp, 
+                (hash_del_f)list_destroy);
 
   if ((errcount = _genders_open_and_parse(handle, 
 					  filename,
 					  debugnodeslist, 
 					  debugattrvalslist,
+                                          &(debugnode_index),
+                                          &(debugnode_index_size),
+                                          &(debugattr_index),
+                                          &(debugattr_index_size),
 					  1, 
 					  stream)) < 0)
     goto cleanup;
@@ -1115,6 +1157,8 @@ genders_parse(genders_t handle, const char *filename, FILE *stream)
  cleanup:
   __list_destroy(debugnodeslist);
   __list_destroy(debugattrvalslist);
+  __hash_destroy(debugnode_index);
+  __hash_destroy(debugattr_index);
   return rv;
 }
 
