@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_test.c,v 1.8 2008/03/28 16:56:32 chu11 Exp $
+ *  $Id: genders_test.c,v 1.9 2009/06/02 18:05:21 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -152,6 +152,7 @@ _test_corner_cases(void)
   errtotal += _corner_case(genders_isattrval_corner_case, "genders_isattrval");
   errtotal += _corner_case(genders_index_attrvals_corner_case, "genders_index_attrvals");
   errtotal += _corner_case(genders_query_corner_case, "genders_query");
+  errtotal += _corner_case(genders_testquery_corner_case, "genders_testquery"); 
   errtotal += _corner_case(genders_parse_corner_case, "genders_parse");
   errtotal += _corner_case(genders_set_errnum_corner_case, "genders_set_errnum");
 
@@ -218,6 +219,7 @@ _test_functionality(void)
   errtotal += _functionality(genders_isattrval_functionality, "genders_isattrval");
   errtotal += _functionality(genders_index_attrvals_functionality, "genders_index_attrvals");
   errtotal += _functionality(genders_query_functionality, "genders_query");
+  errtotal += _functionality(genders_testquery_functionality, "genders_testquery");
   errtotal += _functionality(genders_parse_functionality, "genders_parse");
   errtotal += _functionality(genders_set_errnum_functionality, "genders_set_errnum");
 
